
/*
 * Extended and enhanced by Michael Juntao Yuan and Ju Long for
 * JavaWorld.
 *
 * Sun's original copyright notice is as following:
 *
 * Copyright 1999-2001 Sun Microsystems, Inc. ALL RIGHTS RESERVED
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN
 * OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of
 * any nuclear facility.
 */


package Session;

import java.io.*;

import javax.microedition.io.*;
import javax.microedition.rms.*;

/**
 * Simple way of putting persistent cookie support into the framework. Use
 * RMSCookieConnector.open() instead of Connector.open() 
 */
public class RMSCookieConnector {
    private RMSCookieConnector() {}

    // The default name of the RMS to store cookies.
    private static String cookieStoreName = "cookieStore";
    
    // Use the default RMS cookie store name.
    public static HttpConnection open(String url) throws Exception {
        return open(url, cookieStoreName);
    }

    // Open a new connection and save cookie into the specified store.
    public static HttpConnection open(String url, String storeName) 
                                                      throws Exception {
        cookieStoreName = storeName;
        // Open a new connection.
        HttpConnection c = (HttpConnection) Connector.open(url);
        // Find cookies from the store and add to the connection header.
        addCookie(c, url);
        c.setRequestProperty("User-Agent",
                             "Profile/MIDP-1.0, Configuration/CLDC-1.0");
        c.setRequestProperty("Content-Language",
                             "en-US");
        // Cookie aware wrapper to the connection object.
        HttpConnection sc = new HttpRMSCookieConnection(c);
        return sc;
    }

    // Close the connection and streams.
    public static void close(HttpConnection conn,
			     InputStream is,
			     OutputStream os) {
	if (is != null) {
	    try {
		is.close();
	    } catch (IOException ignore) {
		// ignore
	    }
	}

	if (os != null) {
	    try {
		os.close();
	    } catch (IOException ignore) {
		// ignore
	    }
	}

	if (conn != null) {
	    try {
		conn.close();
	    } catch (IOException ignore) {
		// ignore
	    }
	}
    }

    // Retreive cookies from the connection header and save
    // them into the cookie store.
    //
    // In the store, each cookie is associated with a domain.
    //
    // The record store format is:
    // cookie1, domain1, cookie2, domain2 ...
    static void getCookie(HttpConnection c) throws IOException {
      int k = 0;
      try {
        // Open the record store.
        RecordStore rs = RecordStore.openRecordStore(cookieStoreName, true);
        // Iterate through connection headers and find "set-cookie" fields.
        while (c.getHeaderFieldKey(k) != null) {
          String key = c.getHeaderFieldKey(k);
          String value = c.getHeaderField(k);
          if (key.equals("set-cookie")) {
            // Parse the header and get the cookie.
            int j = value.indexOf(";");
            String cValue = value.substring(0, j);
            // Write the cookie into the cookie store.
            int newID = rs.addRecord(cValue.getBytes(), 0, cValue.length());

            // We set the domain default to the current server.
            String dValue = c.getHost();
            if ( dValue == null ) {
              // If there is no valid domain, 
              // we do not keep the cookie.
              rs.deleteRecord(newID);
            } else {
              // All upper case for easy comparision in the future.
              dValue = dValue.toUpperCase();
              // Write the domain into the cookie store.
              rs.addRecord(dValue.getBytes(), 0, dValue.length());
            }
          }
          k++;
        }
        // Close store.
        rs.closeRecordStore();
      } catch ( Exception e ) {
        throw new IOException( e.getMessage() );
      }
      return;
    }

    // This method matches cookies in the store with the domain
    // of the connection. The matched cookies are set into the
    // headers of the connection.
    static void addCookie(HttpConnection c, String url) throws Exception {

      // String variable domain stores the domain of of the current url.
      String domain;
      // Find the "/" or ":" after "http://"
      int indexSlash = url.indexOf("/", 7); 
      int indexColon = url.indexOf(":", 7); 
      if ( (indexSlash == -1) && (indexColon == -1) ) {
        // If neither is found, domain ends at the end of the URL.
        domain = url.substring(7, url.length());
      } else if ( indexSlash == -1 ) {
        // If not slash is found but a colon is found, the domain
        // name ends at the colon.
        domain = url.substring(7, indexColon);
      } else if ( indexColon == -1 ) {
        // If not colon is found but a slash is found, the domain
        // name ends at the slash.
        domain = url.substring(7, indexSlash);
      } else if ( (indexSlash < indexColon) ) {
        // If both are found but slash appears before colon,
        // the domain name ends at the slash.
        domain = url.substring(7, indexSlash);
      } else {
        // If both are found but colon appears before slash,
        // the domain name ends at the colon.
        domain = url.substring(7, indexColon);
      }
      domain = domain.toUpperCase();

      StringBuffer buff = new StringBuffer();
      RecordStore rs = RecordStore.openRecordStore(cookieStoreName, true);
      RecordEnumeration re = rs.enumerateRecords(null, null, false);
      String cookie = "", cookieDomain = "";
      // Iterate through the cookie record store and find cookies
      // with domain matching the current URL.
      //
      // isCookie is used to tell whether the current record is
      // a cookie or an associated domain.
      boolean isCookie = true;
      while ( re.hasNextElement() ) {
        if ( isCookie ) {
          cookie = new String(re.nextRecord());
        } else {
          cookieDomain = new String(re.nextRecord());
          // Cookies are valid for sub-domains.
          if ( domain.endsWith( cookieDomain ) ) {
            buff.append( cookie );
            buff.append("; ");
          }
        }
        isCookie = !isCookie;
      }
      // Close the store.
      rs.closeRecordStore();

      // If we do have cookies to send, set the composed string into 
      // "cookie" header.
      String cookieStr = buff.toString();
      if ( cookieStr == null || cookieStr.equals("") ) {
        // Ignore
      } else {
        c.setRequestProperty( "cookie", cookieStr );
      }
      return;
    }

    // Remove all cookies.
    public static void removeCookies() throws Exception {
        RecordStore.deleteRecordStore(cookieStoreName);
        return;
    }

}

// Wrapper (decorator) class.
class HttpRMSCookieConnection implements HttpConnection {
    private HttpConnection c;
   
    public HttpRMSCookieConnection(HttpConnection c) {
        this.c = c;
    }
  
    public String getURL() {
        return c.getURL();
    }

    public String getProtocol() {
        return c.getProtocol();
    }

    public String getHost() {
        return c.getHost();
    }

    public String getFile() {
        return c.getFile();
    }

    public String getRef() {
        return c.getRef();
    }
             
    public String getQuery() {
        return c.getQuery();
    }
        
    public int getPort() {
        return c.getPort();
    }
    
    public String getRequestMethod() {
        return c.getRequestMethod();
    }

    public void setRequestMethod(String method) throws IOException {
        c.setRequestMethod(method);
    }

    public String getRequestProperty(String key) {
        return c.getRequestProperty(key);
    }
     
    public void setRequestProperty(String key, String value)
        throws IOException {
        c.setRequestProperty(key, value);
    }

    public int getResponseCode() throws IOException {
        return c.getResponseCode();
    }

    public String getResponseMessage() throws IOException {
        return c.getResponseMessage();
    }

    public long getExpiration() throws IOException {
        return c.getExpiration();
    }

    public long getDate() throws IOException {
        return c.getDate();
    }

    public long getLastModified() throws IOException {
        return c.getLastModified();
    }

    public String getHeaderField(String name) throws IOException {
        return c.getHeaderField(name);
    }

    public int getHeaderFieldInt(String name, int def)
        throws IOException {
        return c.getHeaderFieldInt(name, def);
    }

    public long getHeaderFieldDate(String name, long def)
        throws IOException {
        return c.getHeaderFieldDate(name, def);
    }
    
    public String getHeaderField(int n) throws IOException {
        return c.getHeaderField(n);
    }
    
    public String getHeaderFieldKey(int n) throws IOException  {
        return c.getHeaderFieldKey(n);
    }
    
    public String getType() {
        return c.getType();
    }

    public String getEncoding() {
        return c.getEncoding();
    }

    public long getLength() {
        return c.getLength();
    }

    public void close() throws IOException {
        c.close();
    }

    // The cookies have to be retrieved when we open the input stream.
    public InputStream openInputStream() throws IOException {
        checkResponseCode();
        RMSCookieConnector.getCookie(c);          
        return c.openInputStream();
    }
    
    // The cookies have to be retrieved when we open the input stream.
    public DataInputStream openDataInputStream() throws IOException {
        checkResponseCode();
        RMSCookieConnector.getCookie(c);  
        return c.openDataInputStream();
    }
    
    public OutputStream openOutputStream()
        throws IOException {
        return c.openOutputStream();
    }
    
    public DataOutputStream openDataOutputStream()
        throws IOException {
        return c.openDataOutputStream();
    }

    private void checkResponseCode() throws IOException {
        int code = c.getResponseCode();
        if (code != c.HTTP_OK) {
            throw new IOException(code + "; " + c.getResponseMessage());
        }
    }
}
